# -*- coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import absolute_import
from __future__ import division

import os
import sys

# TODO
DEFAULT_ENCODING = 'utf-8'

# TODO
# SNAPSHOTS_DIR = '/mnt/sataraid1/snapshots'
from product_conf.devinfo import (VOL_INFO,)
SATARAID1_MP = VOL_INFO['sataraid1']['mnt_point'].decode('utf-8')
SNAPSHOTS_DIR = os.path.join(SATARAID1_MP, 'snapshots')


def create_symlinks(bi=None, dsn=None, encoding=DEFAULT_ENCODING):
    backup_jobs = __obtain_backup_jobs()
    sharedfolders = __obtain_sharedfolders()

    def __get_path_from_sharename(sharename):
        for sharedfolder in sharedfolders:
            if sharedfolder['name'] == sharename:
                return sharedfolder['path']
        return None

    for backup_job in backup_jobs:
        if bi and bi != backup_job['id']:
            continue
        s_sharenames = []
        for src in backup_job['srclist']:
            s_type = src['type']
            if s_type != 'local':
                continue
            s_sharename = src['sharename']
            s_sharenames.append(s_sharename)
        d_sharenames = []
        for dst in backup_job['dstlist']:
            d_type = dst['type']
            if d_type != 'local':
                continue
            d_sharename = dst['sharename']
            if dsn and d_sharename != dsn:
                continue
            d_sharenames.append(d_sharename)
        for d_sharename in d_sharenames:
            d_path = __get_path_from_sharename(d_sharename)
            d_job_path = os.path.join(d_path, backup_job['name'])
            for generation in os.listdir(d_job_path.encode(encoding)):
                generation = generation.decode(encoding)
                # FIXME: if generation is not generation
                for s_sharename in s_sharenames:
                    realpath = os.path.join(d_job_path,
                                            generation,
                                            'localhost',
                                            s_sharename)
                    linkpath = os.path.join(_get_snapdir(s_sharename),
                                            _get_snapshot_name(generation))
                    try:
                        os.symlink(realpath.encode(encoding),
                                   linkpath.encode(encoding))
                    except OSError as err:
                        if err.errno == 17:
                            os.remove(linkpath.encode(encoding))
                            os.symlink(realpath.encode(encoding),
                                       linkpath.encode(encoding))
                        else:
                            raise


def clean_snapdirs(encoding=DEFAULT_ENCODING):
    for _sharedfolder_name in os.listdir(SNAPSHOTS_DIR.encode(encoding)):
        _sharedfolder_name = _sharedfolder_name.decode(encoding)
        _snapdir = _get_snapdir(_sharedfolder_name)
        _clean_snapdir(_snapdir)


def _get_snapdir(sharedfolder_name):
    return os.path.join(SNAPSHOTS_DIR, sharedfolder_name)


def _get_snapshot_name(gen):
    YYYY = gen[0:4]
    MM = gen[4:6]
    DD = gen[6:8]
    hh = gen[8:10]
    mm = gen[10:12]
    ss = gen[12:14]
    return '@GMT-{YYYY}.{MM}.{DD}-{hh}.{mm}.{ss}'.format(
        YYYY=YYYY,
        MM=MM,
        DD=DD,
        hh=hh,
        mm=mm,
        ss=ss,
    )


def _clean_snapdir(snapdir, encoding=DEFAULT_ENCODING):
    for symlink_name in os.listdir(snapdir.encode(encoding)):
        symlink_name = symlink_name.decode(encoding)
        symlink_path = os.path.join(snapdir, symlink_name)
        if os.path.islink(symlink_path) and os.path.exists(symlink_path):
            os.unlink(symlink_path)


from ionas.request import Request, Entry, EntryList, XmlConfNode
from product_conf import SUDO_CMD
from fal_info import FAL_BACKUPS_PATH, FAL_SHARES_PATH
def __call(falcmd, entry, raise_exception=False):
    try:
        entrylist = EntryList()
        entrylist.append_entry(entry)
        request = Request(falcmd, filter_cmd=SUDO_CMD)
        response = request.request(entrylist)
        return response.get_entrylist().get_entry()
    except:
        return None
def __obtain_backup_jobs():
    # TODO
    entry = Entry('load', '<backups />')
    entry = __call(FAL_BACKUPS_PATH, entry)
    jobs = []
    backups_node = entry.get()
    backup_node = backups_node.get_node('backup')
    for job_node in backup_node.get_node_list('joblist/job'):
        job = {}
        job['id'] = job_node.get_node('id').get_text().decode('utf-8')
        job['name'] = job_node.get_node('name').get_text().decode('utf-8')
        job['srclist'] = []
        for src_node in job_node.get_node_list('sourcelist/source'):
            src_type = src_node.get_node('type').get_text().decode('utf-8')
            if src_type != 'local':
                continue
            src_sharename = src_node.get_node('sharename').get_text().decode('utf-8')
            if src_sharename == '':
                continue
            job['srclist'].append({
                'type': src_type,
                'sharename': src_sharename,
            })
        job['dstlist'] = []
        for dst_node in job_node.get_node_list('destination'):
            dst_type = dst_node.get_node('type').get_text().decode('utf-8')
            if dst_type != 'local':
                continue
            dst_sharename = dst_node.get_node('sharename').get_text().decode('utf-8')
            if dst_sharename == '':
                continue
            job['dstlist'].append({
                'type': dst_type,
                'sharename': dst_sharename,
            })
        jobs.append(job)
    return jobs
def __obtain_sharedfolders():
    # TODO
    entry = Entry('load', '<shares />')
    entry = __call(FAL_SHARES_PATH, entry)
    sharedfolders = []
    shares_node = entry.get()
    for share_node in shares_node.get_node_list('sharelist/share'):
        name = share_node.get_node('name').get_text().decode('utf-8')
        path = share_node.get_node('path').get_text().decode('utf-8')
        sharedfolders.append({
            'name': name,
            'path': path,
        })
    return sharedfolders


# FIXME
def obtain_sharedfolders():
    return __obtain_sharedfolders()
